/*
    v0.01 - original

todo:
- tap does not inhibit << >> keys
- flips back to bax 2 on aspect ratio change - fixed?
- filter on/off button
- automatically swap to be the first tap
- support for >2 toppies
- differentiate between toppy on vcr scar, and other device [use code on slow blanking]
- more .ini settings
- maybe add option to alow 2nd press to put it in pass through mode
- maybe add option to alow 2nd press to show which toppy is active



1 remote - 2 toppies
--------------------

use 'sat' key to select active box
      ______             ________________             ______________
     |      |           |                |           |              |
     |  tv  +---scart---+tv           vcr+---scart---+tv         vcr+----scart----
     |______|           |________________|           |______________|
                        topfield1                    topfield2

- tap should be first in load order
- tap should run on both toppies
- .ini file sets parameters

- 1st press of tv/stb button, shows status
- 2nd press (within 3 seconds) switches active toppy

toppy1:
- ignores the 'sat' button
- monitors vcr scart
  if it detects no device on the vcr scart, it enables its tv scart output, enables its remote
  if it detects a device on the vcr scart, it sets itself in passthrough mode, disables its remote

toppy 2:
- monitors 'sat' button
  when clear it enables its tv scart output, enables its remote,
  when set   it disables its tv scart output, disables its remote

*/




/*
juwlz .ini suggestion
That comment, plus something one of the MyStuff team said, has prompted the idea
for a ToppySwitcher TAP. (This may be similar to BobD's own solution, but he's
away at the moment, and anyway, there's a lot of MyStuff coding to do.)

The basic idea is that the ToppySwitcher TAP is first in the load order. If the
Toppy is currently enabled it passes on any button other than the Activation
button (TV/Sat by default). If disabled, it swallows everything except the
Activation button.

If it sees the Activation button, it then listens for another button press
within the next <n> seconds, and if it sees something it recognises, it acts
upon it. Only the currently active Toppy will ever display anything. If the
config files are different/conflicting, the config file for the Toppy which is
enabled when the Activation key is pressed should be used to decide how to
switch, and the ini file for the selected Toppy should be used for confirmation.
(Of course, in most cases, both will be the same anyway.)

The Toppy selection button for each machine can be any button on the remote
other than the Activation key or the Menu key (and is defined in ini file).
* Double press of the Activation key confirms which Toppy is currently active
  using the configured confirmation method (if confirmation method is silent, use
  Text display, method 1 for this)
* Activation key followed by Menu key brings up the config menu (if such a
  friendly things is implemented)
* Activation key followed by an enable key for any of the Toppies enables one
  Toppy and disables all others

So, by default, TV/Sat followed by 1 enables Toppy 1, disables Toppy 2, and
displays a message saying "Toppy 1" in black text on a white background one
third of the way down the screen to confirm that Toppy 1 is enabled (i.e.
listening for key presses).

Pressing TV/Sat twice brings up a text message saying "Toppy 1" to confirm that
Toppy 1 is enabled.

Pressing TV/Sat followed by Menu brings up the config screen (of course this
will need to be done on each Toppy, and to prevent confusion, each Toppy's
config should be identical except for the Enable Button for this Toppy in column
2 of line 1).

Sample ini / readme file with default settings:

TV/Sat, 1, Verbose, 1
1, "Toppy 1", White, Black
2, "Toppy 2", Black, White

# Line 1 fields (global config):
# 1. TAP activation button (must be the same on all Toppies)
# 2. Enable button for this Toppy (must be different for each Toppy)
# 3. Switching style (Verbose | Silent)
# 4. Confirmation style
# 5 ... optional Confirmation parameters
#
# Config line for each Toppy
# 1. Enable button (anything except Activation button or Menu button)
# 2. Toppy ID
# 3. Background colour for confirmation
# 4. Foreground colour for confirmation
#
# Global config detail:
# Confirmation style and parameters
#
# 0 - Silent, no confirmation
# 1 - Text (display text in column 2 in Foreground colour text on Background colour
#      Options [Duration, X offset, Y offset]
#      Default is to display centered horizontally, and vertically
        ToppyNum/(NumToppies+1) down the screen (e.g. 1/3 way down for Toppy 1,
        2/3 down for Toppy2, etc.)
#      Offsets of -1 (or omitted) mean "centred"
# 2 - Blob
#      Options [Duration, X offset, Y offset]
#      Default is to display first character of Toppy ID in Foreground colour
#      text in a small circle of Background colour in top right corner.
#      Negative offsets are relative to right/bottom instead of left/top
# 3 - Frame
#      Options [Duration, numpixels]
#      Default is to display a 5 pixel border in background colour
#
# Example:
# TV/Sat, Red, Silent, 3, 2, 10
# Red, "Old Toppy - 160GB", Red, White
# Yellow, "New Toppy - 250GB", Yellow Black
#
# This uses TV/Sat as the activation key, doesn't display options when TV/Sat is
# first pressed, has the Red button as the activation key for this Toppy,
# displays a 10 pixel wide Red frame around the picture for 2 time units to
# confirm that the Red button has been pressed and the Red Toppy has been enabled.
#
# Example:
# TV/Sat, TV/Radio, Verbose, 1
# TV/Radio, "Silver Toppy", White, Black

# Opt, "Black Panther", Black, White
# 0, "Disable all Toppies", Red, Yellow
#
# This uses TV/Sat as the activation key, displays 3 lines of choices when the TV/Sat button is pressed:
#        TV/Radio:  Silver Toppy   (displayed in Black text on White background)
#        Opt:  Black Panther   (displayed in White text on Black background)
#        0:   Disable all Toppies   (displayed in Yellow text on Red background)
#
# When TV/Radio is pressed, it confirms enabled Toppy by displaying "Silver
# Toppy" in Black text on White background, 1/4 of the way down the screen
#
# displays a 10 pixel wide Red frame around the picture for 2 time units to
# confirm that the Red button has been pressed and the Red Toppy has been enabled.
#

Anybody fancy writing it for the growing (but niche) band of multiple Toppy
households? I bet that other than a pretty config screen any of you TAP authors
could knock this up in pretty short order  .

NB: If there's a nice friendly configuration screen, the buttons and colours
could be held in the ini file as the button codes and hex values without making
it too obscure. Of course, that could still be the case even if there isn't a
nice friendly config screen. It would just be less ... friendly.

NB: The default durations, positions and pixel widths may not be appropriate.
I've just picked numbers out of the air. Duration could be counted as seconds,
1/10 seconds, milliseconds, whatever. I defer to the TAP author's better
judgement.

Julie
*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "libFirebird.h"
//#include "fnames1408.h"


#define ID_USERTAP 0xbdb03501
TAP_ID( ID_USERTAP );

TAP_PROGRAM_NAME("remote_sharer v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("a tap to share 1 remote with 2 toppies");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

void TSRCommanderConfigDialog();
bool TSRCommanderExitTAP();
int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//
type_menu main_menu;

int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int main_menu_handler(type_menu *menu, dword param1);
int key_handler(dword param1);

int init(void);
int close(void);

void remote_status(void);
int on_keypress1(int param1);
int on_keypress2(int param1);
int one_second_enabled = 1;
dword one_second = 0;
void one_second_update(void);
int set_fw_addresses();

int stb_timeout = 0;
int stb_state = 0;
//----------------------------------------------------------------------------
int ini_init();
int ini_update();
//----------------------------------------------------------------------------
//default values if no .ini
int box_id = 1;                     //{1,2}
char box_name[0x20] = "Toppy 1";    //
int active = 1;                     //irrelevent
int enable_scart = 0;               //todo
int menu_enable = 0;                //1=enable the menu
int menu_key = RKEY_Recall;         //1=enable the menu key
int debug = 0;                      //1=enable serial debug messages
int splash_enable = 1;              //1=enable this inital splash osd
//----------------------------------------------------------------------------
void *fw_AVMatrixWriteRegister;
void *fw_AVMatrixReadRegister;
void *fw_set_tv_slow_blanking;
byte *fw_av_stb;
byte *fw_av_matrix0;
byte *fw_av_matrix1;
byte *fw_av_matrix2;
byte *fw_av_matrix3;
byte *fw_av_matrix4;
byte *fw_av_matrix5;
dword patch_set_tv_slow_blanking[2];
//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
bool TSRCommanderExitTAP()
{
    close();
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    TAP_ExitNormal();
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
	type_TAPProcess *TAPProcess;

    if (fw_tap_table <= 0) return 0;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
                if (TAPProcess->header->id == 0x81001021) {  //tap_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//print multiline message to OSD
//  500=vertical position(0-576)
//  RGB(3,5,10)=background colour(dark blue)
//  300=hold time(3 seconds)
//  0=blocking
//
void oprintf(const void *fmt, ...)
{
    va_list argp;

    va_start(argp, fmt);
    vsprintf(strLog, fmt, argp);
    if (debug) TAP_Print("\n%s", strLog);
    ShowMessage_y(strLog, 500, RGB(3,5,10), 300, 0);
    va_end(argp);

    return;
}

void xyprintf(int x, int y, const void *fmt, ...)
{
    va_list argp;

    va_start(argp, fmt);
    vsprintf(strLog, fmt, argp);
    if (debug) TAP_Print("\n%s", strLog);
    ShowMessage_xy(strLog, x, y, COLOR_White, RGB(3,5,10), 100, 0);
    va_end(argp);

    return;
}
//-----------------------------------------------------------------------------
/*
    box_id
    box_name
    active
    enable_scart
    menu_enable
    menu_key
    debug
    splash_enable

*/
int ini_init()
{
    int ini_location;
    char str[0x20];

    ini_location = INIOpenFile("remote_sharer.ini", "remote_sharer");
    box_id = INIGetInt("box_id", box_id, 0x80000000, 0x7fffffff);
    INIGetString("box_name", box_name, box_name, sizeof(box_name));
    active = INIGetInt("active", active, 0x80000000, 0x7fffffff);
    enable_scart = INIGetInt("enable_scart", enable_scart, 0x80000000, 0x7fffffff);
    menu_enable = INIGetInt("menu_enable", menu_enable, 0x80000000, 0x7fffffff);
    menu_key = INIGetHexDWord("menu_key", menu_key, 0x00000000, 0xffffffff);
    debug = INIGetHexDWord("debug", debug, 0x00000000, 0xffffffff);
    splash_enable = INIGetInt("splash_enable", splash_enable, 0x80000000, 0x7fffffff);

    INICloseFile();

    ini_update();
}

int ini_update()
{
    int ini_location;

    ini_location = INIOpenFile("remote_sharer.ini", "remote_sharer");
    INISetInt("box_id", box_id);
    INISetString("box_name", box_name);
    INISetInt("active", active);
    INISetInt("enable_scart", enable_scart);
    INISetInt("menu_enable", menu_enable);
    INISetHexDWord("menu_key", menu_key);
    INISetHexDWord("debug", debug);
    INISetInt("splash_enable", splash_enable);

    INISaveFile("remote_sharer.ini", INILOCATION_AtSettings, "remote_sharer");
    INICloseFile();
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {2, "xxx",                 },
        {0, "Status",              },
        {1, "Exit Tap",            },
    },
};

//----------------------------------------------------------------------------
int TAP_Main(void)
{
    int ret;

    tap_name =  (char *)__tap_program_name__;
    if (debug) sprintf(strLog, "Opening %s", tap_name);
    if (debug) log_message(LOG_SERIAL, strLog, DELAY_SHORT);

    //determine support for different firmwares
    ret = exTAP_Init();
    if (ret < 0) {
        sprintf(strLog, "Unsupported platform - %d:%04x", fw_sysid, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
        return 0;
    }
    ret = set_fw_addresses();
    if (ret < 0) {
        sprintf(strLog, "Unsupported platform - %d:%04x", fw_sysid, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
        return 0;
    }

    init();
    if (splash_enable) sprintf(strLog, "Opening %s", tap_name);
    if (splash_enable) log_message(LOG_SCREEN_CONT, strLog, DELAY_SHORT);

    one_second = TAP_GetTick(); + (1*100);
    return 1;                 //0=application mode, 1=tsr mode
}


char menu_title_str[0x40];

int init(void)
{
    int next_box_id;

    main_menu.visible = 0;

    ini_init();

    sprintf(menu_title_str, "%s - %s", box_name, tap_name);
    main_menu.fixme_titlep = menu_title_str - _tap_startAddr;

    TSRCommanderInit(0, TRUE);
    return 0;
}

int close(void)
{

    DeleteMenu(&main_menu);

    //ensure blanking patch is cleared
    if (patch_set_tv_slow_blanking[1]) {
        *(dword *)((dword)fw_set_tv_slow_blanking+0) = patch_set_tv_slow_blanking[0];
        *(dword *)((dword)fw_set_tv_slow_blanking+4) = patch_set_tv_slow_blanking[1];
    }

    if (splash_enable) sprintf(strLog, "Closing  %s", tap_name);
    if (splash_enable) log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (message_rgn) TAP_Osd_Delete(message_rgn);
    if (OpLog) TAP_Hdd_Fclose(OpLog);
    TSRCommanderErase();
    if (debug) TAP_Print("\nClosing %s", tap_name);
    return 0;
}

//----------------------------------------------------------------------------
dword TAP_EventHandler(word event, dword param1, dword param2)
{
    dword tick;
    dword state, subState;

    switch (event) {
        case EVT_IDLE:
//            TAP_Print("\nEVT_IDLE, event=%x, param1=%x, param2=%x", event, param1, param2);
            tick = TAP_GetTick();

            if (message_showing) {
                if (tick > message_showing) {
                    TAP_EnterNormal();
                    TAP_Osd_Delete(message_rgn);
                    message_showing = 0;
                    message_rgn = 0;
                }
            }

            if (one_second_enabled) {
                if (tick > one_second) {
                    one_second_update();
//                    one_second = tick + (1*100);
                    one_second = tick + (1*20);
                }
            }

            if (stb_timeout) {
                if (tick > stb_timeout) {
//                  if (debug) printf("\nmessage_showing=%x, stb_state=%x, active=%x", message_showing, stb_state, active);
                    if (message_showing) {
                        TAP_EnterNormal();
                        TAP_Osd_Delete(message_rgn);
                        message_showing = 0;
                        message_rgn = 0;
                    }
                    if (stb_state > 0) {
                        stb_state = 0;
                        stb_timeout = 0;
                    }
                }
            }

    		TSRCommanderWork();
            break;

        case EVT_KEY:
//            TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);
                TAP_GetState(&state, &subState);
//            TAP_Print("\nEVT_KEY, state=%x, subState=%x", state, subState);

            if (box_id==1) {
                param1 = on_keypress1(param1);
            }
            if (box_id==2) {
                param1 = on_keypress2(param1);
            }

            if (menu_enable) {
                if (main_menu.visible) {
                    param1 = main_menu_handler(&main_menu, param1);
                }
                else
                if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                    param1 = key_handler(param1);
                }
            }
            break;

        default:
//            TAP_Print("\nEVT_???, event=%x, param1=%x, param2=%x", event, param1, param2);
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            TAP_EnterNormal();
            switch (selection) {
                case 0:
                    remote_status();
                    return 0;

                case 1:
                    close();
                    TAP_Exit();
                    return 0;
            }
            break;
        case RKEY_Exit:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return 0;
        default:
            if (param1 == menu_key) {
                 DeleteMenu(menu);
                 TAP_EnterNormal();
                 return param1;
            }
            TAP_Win_Action(&menu->win, param1);
            return 0;
    }
    return param1;
}

int key_handler(dword param1)
{
    if (param1 == menu_key) {
        //only if tsrcommander is not running
        if (!is_tsr_commander_running()) {
            TAP_ExitNormal();
            ShowMenu(&main_menu);
            return 0;
        }
    }
    return param1;
}

//----------------------------------------------------------------------------
int set_fw_addresses()
{
    if (fw_sysid == 456) {
        switch(_appl_version) {
            case 0x1408:
                fw_AVMatrixWriteRegister = (void *)0x800b030c;
                fw_AVMatrixReadRegister  = (void *)0x800b04e4;
                fw_set_tv_slow_blanking  = (void *)0x800890e4;
                fw_av_stb                = (byte *)(fw_gp-0x620c);      //0=vid, 1=enc
                fw_av_matrix0            = (byte *)(fw_gp-0x64e8);      //shadow copy of av_matrix registers
                fw_av_matrix1            = fw_av_matrix0 + 1;
                fw_av_matrix2            = fw_av_matrix0 + 2;
                fw_av_matrix3            = fw_av_matrix0 + 3;
                fw_av_matrix4            = fw_av_matrix0 + 4;
                fw_av_matrix5            = fw_av_matrix0 + 5;
                break;
            case 0x1340:
                fw_AVMatrixWriteRegister = (void *)0x800aec6c;
                fw_AVMatrixReadRegister  = (void *)0x800aee44;
                fw_set_tv_slow_blanking  = (void *)0x800884c0;
                fw_av_stb                = (byte *)(fw_gp-0x6293);      //0=vid, 1=enc
                fw_av_matrix0            = (byte *)(fw_gp-0x6560);      //shadow copy of av_matrix registers
                fw_av_matrix1            = fw_av_matrix0 + 1;
                fw_av_matrix2            = fw_av_matrix0 + 2;
                fw_av_matrix3            = fw_av_matrix0 + 3;
                fw_av_matrix4            = fw_av_matrix0 + 4;
                fw_av_matrix5            = fw_av_matrix0 + 5;
                break;
            case 0x1365:
                fw_AVMatrixWriteRegister = (void *)0x800af134;
                fw_AVMatrixReadRegister  = (void *)0x800af30c;
                fw_set_tv_slow_blanking  = (void *)0x80088898;
                fw_av_stb                = (byte *)(fw_gp-0x628B);      //0=vid, 1=enc
                fw_av_matrix0            = (byte *)(fw_gp-0x6560);      //shadow copy of av_matrix registers
                fw_av_matrix1            = fw_av_matrix0 + 1;
                fw_av_matrix2            = fw_av_matrix0 + 2;
                fw_av_matrix3            = fw_av_matrix0 + 3;
                fw_av_matrix4            = fw_av_matrix0 + 4;
                fw_av_matrix5            = fw_av_matrix0 + 5;
                break;
            default:
                return -1;
        }
        //save the original
        if (*(dword *)((dword)fw_set_tv_slow_blanking+4)) {
            patch_set_tv_slow_blanking[0] = *(dword *)((dword)fw_set_tv_slow_blanking+0);
            patch_set_tv_slow_blanking[1] = *(dword *)((dword)fw_set_tv_slow_blanking+4);
        }
        else {
            patch_set_tv_slow_blanking[0] = 0;
            patch_set_tv_slow_blanking[1] = 0;
        }
        return 0;
    }
    return -1;
}

//----------------------------------------------------------------------------
//
//monitor the slow blanking input (pin 8) on the vcr scart
//0x=low, 10=5v=16:9, 11=10v=4:3
//
void one_second_update(void)
{
    dword tick;
    dword slow_blanking;
    byte av_stb;

    slow_blanking = exTAP_CallFirmware((void *)fw_AVMatrixReadRegister, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    av_stb = *fw_av_stb;      //0=vid, 1=enc
//  if (debug) printf("\nav slow blanking: %02x, av_stb=%x, active=%x", slow_blanking, av_stb, active);

    if (box_id==1) {
        if (active) {
            //if pin8 is high, something is coming in the vcr scart, switch us off
            if (slow_blanking & 8) {
                //set to passthrough vcr scart to tv scart
                if (av_stb) exTAP_CallFirmware((void *)fw_add_task_queue, 0xE300, RKEY_TvSTB, 0, 0, 0, 0, 0, 0, 0, 0, 0);
                //disable the remote
                active = 0;
                //warn the user if he is still seeing a picture
                TAP_ExitNormal();
                xyprintf(50, 50, "%s: Remote Inactive", box_name);
            }
        }
        else {
            //if pin8 is low, nothing is coming in the vcr scart, switch us on
            if ((slow_blanking & 8) == 0) {
                //clear the passthrough, drive the tv scart
                if (av_stb) exTAP_CallFirmware((void *)fw_add_task_queue, 0xE300, RKEY_TvSTB, 0, 0, 0, 0, 0, 0, 0, 0, 0);
                //enable the remote
                active = 1;
                //welcome back the user
                TAP_ExitNormal();
                xyprintf(50, 50, "%s", box_name);
                message_showing = 0xffffffff;
                tick = TAP_GetTick();
                stb_timeout = tick+300;
            }
        }
    }

    if (box_id==2) {
        if (active) {
        }
        else {
            //if the inactive program changees its aspect ratio, the scart gets set
            //hack: put it back
            //better: inhibit it
            if (slow_blanking & 2) {
printf("\noops box 2 has changed ...");
                //suppress tv_stb, disable scart, disable remote
                slow_blanking = *fw_av_matrix5;
                slow_blanking = slow_blanking & 0xcf;
                slow_blanking = slow_blanking | 0x10;
                *fw_av_matrix5 = slow_blanking;
                exTAP_CallFirmware((void *)fw_AVMatrixWriteRegister, 5, (dword)slow_blanking, 0, 0, 0, 0, 0, 0, 0, 0, 0);
            }
        }
    }
}


//toppy1
int on_keypress1(int param1)
{
    dword tick;
    dword state, subState;

    TAP_GetState(&state, &subState);

    if (active) {
        //suppress tv_stb
        if (param1 == RKEY_TvSTB) {
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                TAP_ExitNormal();
                xyprintf(50, 50, "%s", box_name);
                message_showing = 0xffffffff;
                tick = TAP_GetTick();
                stb_timeout = tick+300;
            }
            param1 = 0;
        }
    }
    else {
        if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
            TAP_ExitNormal();
            xyprintf(50, 50, "%s Remote Inactive", box_name);
        }
        //supress everything
        param1 = 0;
    }

    return param1;
}

//toppy2
int on_keypress2(int param1)
{
    dword tick;
    dword state, subState;
    byte slow_blanking;

    TAP_GetState(&state, &subState);
//  if (debug) printf("\nstate=%x, subState=%x, param1=%x, stb_state=%x, active=%x", state, subState, param1, stb_state, active);

    //if inactive, only care about RKEY_TvSTB
    if (param1 != RKEY_TvSTB) {
        if (active == 0) {
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                TAP_ExitNormal();
                xyprintf(50, 50, "%s: Remote Inactive", box_name);
            }
            return 0;
        }
    }

    //1st press - status
    //2nd press, before timeout - change stb
    //
    if (param1 == RKEY_TvSTB) {
        tick = TAP_GetTick();
        switch (stb_state) {
            case 0:
                if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                    //1st press - status
                    TAP_ExitNormal();
                    xyprintf(50, 50, "%s", box_name);
                    message_showing = 0xffffffff;
                    stb_timeout = tick+300;
                    stb_state = 1;
                }
                break;

            case 1:
                //2nd press, before timeout - change stb
                if (active) {
                    //suppress tv_stb, disable scart, disable remote
                    slow_blanking = *fw_av_matrix5;
                    slow_blanking = slow_blanking & 0xcf;
                    slow_blanking = slow_blanking | 0x10;
                    *fw_av_matrix5 = slow_blanking;
                    exTAP_CallFirmware((void *)fw_AVMatrixWriteRegister, 5, (dword)slow_blanking, 0, 0, 0, 0, 0, 0, 0, 0, 0);
                    active = 0;

                    //inhibit further blanking changes by firmware
                    if (patch_set_tv_slow_blanking[1]) {
                        *(dword *)((dword)fw_set_tv_slow_blanking+0) = 0x03e00008;  //03 e0 00 08   jr $ra
                        *(dword *)((dword)fw_set_tv_slow_blanking+4) = 0x00000000;  //00 00 00 00   nop
                    }
                }
                else {
                    //ensure blanking patch is cleared
                    if (patch_set_tv_slow_blanking[1]) {
                        *(dword *)((dword)fw_set_tv_slow_blanking+0) = patch_set_tv_slow_blanking[0];   //27 BD FF E0   addiu $sp, -0x20
                        *(dword *)((dword)fw_set_tv_slow_blanking+4) = patch_set_tv_slow_blanking[1];   //AF B5 00 14   sw    $s5, 0x14($sp)
                    }

                    //enable tv scart out, try to get the aspect ratio right, enable remote
                    slow_blanking = *fw_av_matrix5;
                    slow_blanking = slow_blanking & 0xcf;
                    exTAP_update_aspect_ratio();
                    if (fw_aspect[CHANNEL_Main].ratio==2) { //ASPECT_4_3
                        slow_blanking = slow_blanking | 0x30;
                    }
                    else {
                        slow_blanking = slow_blanking | 0x20;
                    }
                    *fw_av_matrix5 = slow_blanking;
                    exTAP_CallFirmware((void *)fw_AVMatrixWriteRegister, 5, (dword)slow_blanking, 0, 0, 0, 0, 0, 0, 0, 0, 0);

                    active = 1;
                    TAP_ExitNormal();
                    xyprintf(50, 50, "%s", box_name);
                    message_showing = 0xffffffff;
                }
                //inhibit for a while
                stb_timeout = tick+300;
                stb_state = 2;
                break;

            case 2:
                //inhibited
                break;
        }
        param1 = 0;
    }

    return param1;
}

//----------------------------------------------------------------------------
void remote_status(void)
{
    type_etc_structure *etc;
    int slow_blanking;
    dword tick;

    slow_blanking = exTAP_CallFirmware((void *)fw_AVMatrixReadRegister, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    etc = (type_etc_structure *)*(dword *)fw_etc_structure;

    if (debug) {
        printf("\nbox_id                 : %d", box_id);
        printf("\nbox_name               : %s", box_name);
        printf("\nactive                 : %d", active);
        printf("\nenable_scart           : %d", enable_scart);
        printf("\nmenu_enable            : %d", menu_enable);
        printf("\nmenu_key               : %x", menu_key);
        printf("\ndebug                  : %x", debug);
        printf("\nsplash_enable          : %d", splash_enable);

        printf("\n");
        printf("\nav_matrix - STV6412A");
        printf("\n    av_0               : %02x", *fw_av_matrix0);
        printf("\n    av_1               : %02x", *fw_av_matrix1);
        printf("\n    av_2               : %02x", *fw_av_matrix2);
        printf("\n    av_3               : %02x", *fw_av_matrix3);
        printf("\n    av_4               : %02x", *fw_av_matrix4);
        printf("\n    av_5               : %02x", *fw_av_matrix5);
        printf("\nav                     : %02x", slow_blanking);

        printf("\n");
        printf("\nav_stb                 : %02x", *fw_av_stb);

        printf("\n");
        printf("\netc");
        printf("\netc->byte_0b           : %02x", etc->byte_0b);
        printf("\n    Scart Output Source: %x", (etc->byte_0b >> 3) & 3);
        printf("\netc->byte_13           : %02x", etc->byte_13);
        printf("\n    Scart Type         : %x", (etc->byte_13 >> 5) & 1);
    }

    TAP_ExitNormal();
    xyprintf(200, 400, "box_id: %d, box_name: %s\nscart mode: %s\nvcr scart: %s\ntv  scart: %s", box_id, box_name,
        (*fw_av_stb) ?"pass through" :  "output",
        ((slow_blanking & 0x0c) == 0xc)?"4:3" :
        ((slow_blanking & 0x0c) == 0x8)?"16:9" :
        ((slow_blanking & 0x0c) == 0x4)?"low" : "???",
        ((slow_blanking & 0x03) == 0x3)?"4:3" :
        ((slow_blanking & 0x03) == 0x2)?"16:9" :
        ((slow_blanking & 0x03) == 0x1)?"low" : "???");


    message_showing = 0xffffffff;
    tick = TAP_GetTick();
    stb_timeout = tick+300;
}

/*
av_matrix - STV6412A
0, 0x00     0x00    audio   - stereo, 0dB, 0dB, soft                                                           lbu     $t9, -0x64E8($gp)
1, 0x19     0x22    audio   - stereo,encoderL/R,tv_gain,encoderL/R              stereo,tvL/R,0dB,vcrL/R        lbu     $t8, -0x64E7($gp)
2, 0x92     0x33    video   - cvbs_enc, yc_enc                                  tv_mute,cvbs_vcr               lbu     $t7, -0x64E6($gp)
3, 0xd5     0xfb    video   - +3dB,+5dB,rgb_enc,fb_high                         +3dB,+3dB,rgb_vcr,fb_vcr       lbu     $t9, -0x64E5($gp)
4, 0x10     0x50    misc    - sb_norm, vcr_active                               sb_vcr, vcr_active             lbu     $t9, -0x64E4($gp)
5, 0x38     0x00    misc    -                                                   4:3                            lbu     $t7, -0x64E3($gp)
6,                  stbby

read
xxxx_vcr_tv
*/
//----------------------------------------------------------------------------
